//==============================================================================
//
//   AbstractNode.java
//
//   Copyright (c) 2001-2003 Graffiti Team, Uni Passau
//
//==============================================================================
// $Id: AbstractNode.java,v 1.3 2004/05/18 13:20:01 schwoebb Exp $

package org.graffiti.graph;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.logging.Logger;

import org.graffiti.attributes.CollectionAttribute;

import org.graffiti.util.MultipleIterator;

/**
 * Abstract class <code>AbstractNode</code> common functionality for
 * <code>Node</code> implementations.
 *
 * @version $Revision: 1.3 $
 *
 * @see AdjListNode
 */
public abstract class AbstractNode
    extends AbstractGraphElement
    implements Node
{
    //~ Static fields/initializers =============================================

    /** The logger for the current class. */
    private static final Logger logger = Logger.getLogger(AbstractNode.class.getName());

    //~ Constructors ===========================================================

    /**
     * Constructs a new <code>AbstractNode</code>. Sets the graph of the new
     * <code>AbstractNode</code>.
     *
     * @param graph the <code>Graph</code> the new <code>AbstractNode</code>
     *        instance shall belong to.
     */
    public AbstractNode(Graph graph)
    {
        super(graph);
    }

    /**
     * Constructs a new <code>AbstractNode</code>. Sets the graph of the new
     * <code>AbstractNode</code>.
     *
     * @param graph the <code>Graph</code> the new <code>AbstractNode</code>
     *        instance shall belong to.
     * @param coll the <code>CollectionAttribute</code> of the newly created
     *        <code>AbstractNode</code> instance.
     */
    public AbstractNode(Graph graph, CollectionAttribute coll)
    {
        super(graph, coll);
    }

    //~ Methods ================================================================

    /**
     * @see org.graffiti.graph.Node#getAllInEdges()
     */
    public Collection getAllInEdges()
    {
        Collection col = getDirectedInEdges();
        col.addAll(getUndirectedEdges());

        return col;
    }

    /**
     * @see org.graffiti.graph.Node#getAllInNeighbors()
     */
    public Collection getAllInNeighbors()
    {
        Collection col = getInNeighbors();
        col.addAll(getUndirectedNeighbors());

        return col;
    }

    /**
     * @see org.graffiti.graph.Node#getAllOutEdges()
     */
    public Collection getAllOutEdges()
    {
        Collection col = getDirectedOutEdges();
        col.addAll(getUndirectedEdges());

        return col;
    }

    /**
     * @see org.graffiti.graph.Node#getAllOutNeighbors()
     */
    public Collection getAllOutNeighbors()
    {
        Collection col = getOutNeighbors();
        col.addAll(getUndirectedNeighbors());

        return col;
    }

    /**
     * Returns a collection containing the directed, ingoing edges of the
     * <code>Node</code>.
     *
     * @return a collection containing the directed, ingoing edges of the
     *         <code>Node</code>.
     */
    public Collection getDirectedInEdges()
    {
        ArrayList l = new ArrayList();

        for(Iterator it = getDirectedInEdgesIterator(); it.hasNext();)
        {
            l.add((it.next()));
        }

        return l;
    }

    /**
     * Returns a collection containing the directed outgoing edges of the
     * <code>Node</code>.
     *
     * @return a collection containing the directed outgoing edges of the
     *         <code>Node</code>.
     */
    public Collection getDirectedOutEdges()
    {
        List l = new ArrayList();

        for(Iterator it = getDirectedOutEdgesIterator(); it.hasNext();)
        {
            l.add((it.next()));
        }

        return l;
    }

    /**
     * Returns a collection containing all the ingoing and outgoing directed
     * and undirected edges of the current <code>Node</code>. Ingoing and
     * outgoing edges will not be separated and there will be no ordering on
     * the collection.
     *
     * @return a collection containing all ingoing and outgoing directed and
     *         undirected edges of the current <code>Node</code>.
     */
    public Collection getEdges()
    {
        Collection c = new LinkedList();

        for(Iterator itr = getEdgesIterator(); itr.hasNext();)
        {
            c.add(itr.next());
        }

        return c;
    }

    /**
     * Returns the in-degree of the current <code>Node</code>. The in-degree is
     * defined as the number of ingoing, directed edges plus the number of
     * undirected edges.
     *
     * @return the in-degree of the current <code>Node</code>.
     */
    public int getInDegree()
    {
        return getDirectedInEdges().size() + getUndirectedEdges().size();
    }

    /**
     * Returns a collection containing the neighbor nodes belonging to incoming
     * edges. The number of Elements returned by this function might be less
     * than the number returned by <code>getInDegree()</code>, due to the fact
     * that multiple edges between the same two nodes are possible but the
     * corresponding <code>Node</code> is only inserted once.
     *
     * @return a collection containing the neighbor nodes belonging to incoming
     *         edges.
     */
    public Collection getInNeighbors()
    {
        Set s = new HashSet(getUndirectedNeighbors());

        for(Iterator it = getDirectedInEdgesIterator(); it.hasNext();)
        {
            s.add(((Edge) (it.next())).getSource());
        }

        return s;
    }

    /**
     * Returns an iterator over the neighbor nodes belonging to incoming edges.
     *
     * @return an iterator over the neighbor nodes belonging to incoming edges.
     */
    public Iterator getInNeighborsIterator()
    {
        return new MultipleIterator(getInNeighbors().iterator());
    }

    /**
     * Returns a collection containing all the neighbor nodes of the current
     * <code>Node</code>. A neighbor <code>Node</code> is either the source or
     * the target of either an ingoing, outgoing or an undirected
     * <code>Edge</code> of this <code>Node</code>.
     *
     * @return a collection containing all the neighbor nodes of the current
     *         <code>Node</code>.
     */
    public Collection getNeighbors()
    {
        Set s = new HashSet();
        Edge e;

        for(Iterator it = getEdgesIterator(); it.hasNext();)
        {
            e = (Edge) (it.next());

            if(this == e.getSource())
            {
                s.add(e.getTarget());
            }
            else
            {
                s.add(e.getSource());
            }
        }

        return s;
    }

    /**
     * Returns an interator over the neighbor nodes of the current
     * <code>Node</code>. A neighbor nodes is either the source or the target
     * of either an ingoing or outgoing or undirected <code>Edge</code>.
     *
     * @return an iterator over the neighbor nodes of the current
     *         <code>Node</code>.
     */
    public Iterator getNeighborsIterator()
    {
        return new MultipleIterator(getNeighbors().iterator());
    }

    /**
     * Returns the out-degree of the current <code>Node</code>. The out-degree
     * is defined as the number of outgoing, directed edges plus the number of
     * undirected edges.
     *
     * @return the out-degree of the current <code>Node</code>.
     */
    public int getOutDegree()
    {
        return getDirectedOutEdges().size() + getUndirectedEdges().size();
    }

    /**
     * Returns a collection conataining all the neighbors of the current
     * <code>Node</code> which are connected by an outgoing <code>Edge</code>.
     * The number of Elements returned by this function might be less than the
     * number returned by <code>getOutDegree()</code>, due to the fact that
     * multiple edges between the same two nodes are possible but the
     * corresponding <code>Node</code> is only inserted once.
     *
     * @return a collection containing all the neighbor nodes of the current
     *         <code>Node</code> connected by an outgoing <code>Edge</code>.
     */
    public Collection getOutNeighbors()
    {
        Set s = new HashSet(getUndirectedNeighbors());

        for(Iterator it = getDirectedOutEdgesIterator(); it.hasNext();)
        {
            s.add(((Edge) (it.next())).getTarget());
        }

        return s;
    }

    /**
     * Returns an iterator conataining all the neighbors of the current
     * <code>Node</code> which are connected by an outgoing <code>Edge</code>.
     *
     * @return a Iterator containing all the neighbor nodes of the current
     *         <code>Node</code> connected by an outgoing <code>Edge</code>.
     */
    public Iterator getOutNeighborsIterator()
    {
        return new MultipleIterator(getOutNeighbors().iterator());
    }

    /**
     * Returns a collection containing the undirected edges of the
     * <code>Node</code>.
     *
     * @return a collection containing the undirected edges of the
     *         <code>Node</code>.
     */
    public Collection getUndirectedEdges()
    {
        List l = new ArrayList();

        for(Iterator it = getUndirectedEdgesIterator(); it.hasNext();)
        {
            l.add((it.next()));
        }

        return l;
    }

    /**
     * Returns a collection containing all the neighbors which are connected to
     * the current <code>Node</code> by an undirected <code>Edge</code>.
     *
     * @return a collection containing all the neighbors which are connected to
     *         the current <code>Node</code> by an undirected
     *         <code>Edge</code>.
     */
    public Collection getUndirectedNeighbors()
    {
        Set s = new HashSet();
        Edge e;

        for(Iterator it = getUndirectedEdgesIterator(); it.hasNext();)
        {
            e = (Edge) (it.next());

            if(this == e.getSource())
            {
                s.add(e.getTarget());
            }
            else
            {
                s.add(e.getSource());
            }
        }

        return s;
    }
}

//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
