//==============================================================================
//
//   Selection.java
//
//   Copyright (c) 2001-2003 Graffiti Team, Uni Passau
//
//==============================================================================
// $Id: Selection.java,v 1.3 2004/05/18 07:01:57 pausch Exp $

package org.graffiti.selection;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.graffiti.attributes.FieldAlreadySetException;

import org.graffiti.graph.Edge;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;

/**
 * Contains selected nodes and edges.
 * 
 * <p></p>
 * 
 * <p>
 * Even if there are fundamental changes to the selection, don't use something
 * like that:
 * </p>
 * 
 * <p>
 * <code>Selection newSel = new Selection(SelectionModel.ACTIVE);
 * editorSession. getSelectionModel().add(SelectionModel.ACTIVE);
 * editorSession.
 * getSelectionModel().setActiveSelection(SelectionModel.ACTIVE); </code>
 * </p>
 * 
 * <p>
 * Instead, remove all entries within the selection by calling
 * <code>clear()</code> on the active selection (<code>editorSession.
 * getSelectionModel().getActiveSelection()</code>) and add the new selection
 * elements to it. After all changes have been made and the system should be
 * updated, call <code>editorSession.getSelectionmodel().
 * selectionChanged()</code>
 * </p>
 *
 * @version $Revision: 1.3 $
 */
public class Selection
{
    //~ Instance fields ========================================================

    /**
     * The list of selected edges.
     *
     * @see org.graffiti.graph.Edge
     */
    private List edges;

    /**
     * The list of selected nodes.
     *
     * @see org.graffiti.graph.Node
     */
    private List nodes;

    /**
     * Map of graph elements that changed state from unmarked to marked. This
     * map is cleared after a selectionChanged event has been fired.
     */
    private Map newMarked;

    /**
     * Map of graph elements that changed state from marked to unmarked. This
     * map is cleared after a selectionChanged event has been fired.
     */
    private Map newUnmarked;

    /** The name of this selection. */
    private String name;

    //~ Constructors ===========================================================

    /**
     * Constructs a new <code>Selection</code> instance with the given name.
     */
    public Selection()
    {
        this.nodes = new LinkedList();
        this.edges = new LinkedList();
        this.newMarked = new HashMap();
        this.newUnmarked = new HashMap();
    }

    /**
     * Constructs a new <code>Selection</code> instance with the given name.
     *
     * @param name the name of this selection.
     */
    public Selection(String name)
    {
        this.name = name;
        this.nodes = new LinkedList();
        this.edges = new LinkedList();
        this.newMarked = new HashMap();
        this.newUnmarked = new HashMap();
    }

    //~ Methods ================================================================

    /**
     * Returns the list of selected edges.
     *
     * @see org.graffiti.graph.Edge
     */
    public List getEdges()
    {
        return this.edges;
    }

    /**
     * Returns a list containing all edges and nodes in this selection.
     *
     * @return a list containing all edges and nodes in this selection.
     */
    public List getElements()
    {
        List all = new LinkedList();
        all.addAll(this.edges);
        all.addAll(this.nodes);

        return all;
    }

    /**
     * Returns <code>true</code> if no nodes or edges are selected.
     *
     * @return <code>true</code> if no nodes or edges are selected.
     */
    public boolean isEmpty()
    {
        return (this.nodes.isEmpty() && this.edges.isEmpty());
    }

    /**
     * Sets the name.
     *
     * @param name The name to set
     *
     * @throws FieldAlreadySetException DOCUMENT ME!
     */
    public void setName(String name)
        throws FieldAlreadySetException
    {
        if(this.name == null)
        {
            this.name = name;
        }
        else
        {
            throw new FieldAlreadySetException(
                "Name of a selection may not be changed. Create new one.");
        }
    }

    /**
     * Returns the name of this selection.
     *
     * @return the name of this selection.
     */
    public String getName()
    {
        return this.name;
    }

    /**
     * Sets the list of graph elements that have been marked but the  selection
     * listeners have not yet been notified.
     *
     * @param newMarked list of newly marked graph elements
     */
    public void setNewMarked(Map newMarked)
    {
        this.newMarked = newMarked;
    }

    /**
     * Returns the map holding graph elements that have been marked since the
     * last selectionChanged event.
     *
     * @return map holding graph elements that have been marked since the last
     *         selectionChanged event.
     */
    public Map getNewMarked()
    {
        return newMarked;
    }

    /**
     * Sets the list of graph elements that have been unmarked but the
     * selection listeners have not yet been notified.
     *
     * @param newUnmarked list of newly unmarked graph elements
     */
    public void setNewUnmarked(Map newUnmarked)
    {
        this.newUnmarked = newUnmarked;
    }

    /**
     * Returns the map holding graph elements that have been unmarked since the
     * last selectionChanged event.
     *
     * @return the map holding graph elements that have been unmarked since the
     *         last selectionChanged event.
     */
    public Map getNewUnmarked()
    {
        return newUnmarked;
    }

    /**
     * Returns the list of selected nodes.
     *
     * @see org.graffiti.graph.Node
     */
    public List getNodes()
    {
        return this.nodes;
    }

    /**
     * Adds the given node or edge to the selection.
     *
     * @param ge the node or edge to add to the selection.
     */
    public void add(GraphElement ge)
    {
        if(ge instanceof Node)
        {
            this.add((Node) ge);
        }
        else
        {
            this.add((Edge) ge);
        }
    }

    /**
     * Adds the given node to the list of selected nodes.
     *
     * @param node the node to add to the list of selected nodes.
     */
    public void add(Node node)
    {
        this.nodes.add(node);

        newUnmarked.remove(node);
        this.newMarked.put(node, null);
    }

    /**
     * Adds the given edge to the list of selected edges.
     *
     * @param edge the edge to add to the list of selected edges.
     */
    public void add(Edge edge)
    {
        this.edges.add(edge);

        newUnmarked.remove(edge);
        this.newMarked.put(edge, null);
    }

    /**
     * Adds all (graph)elements of the given collection to this selection.
     *
     * @param newElements
     */
    public void addAll(Collection newElements)
    {
        for(Iterator it = newElements.iterator(); it.hasNext();)
        {
            add((GraphElement) it.next());
        }
    }

    /**
     * Adds all elements from the given selection to this selection.
     *
     * @param sel
     */
    public void addSelection(Selection sel)
    {
        for(Iterator it = sel.getElements().iterator(); it.hasNext();)
        {
            add((GraphElement) it.next());
        }
    }

    /**
     * Remove all elements from this selection.
     */
    public void clear()
    {
        for(Iterator it = nodes.iterator(); it.hasNext();)
        {
            newUnmarked.put(it.next(), null);
        }

        for(Iterator it = edges.iterator(); it.hasNext();)
        {
            newUnmarked.put(it.next(), null);
        }

        nodes = new LinkedList();
        edges = new LinkedList();
        newMarked = new HashMap();
    }

    /**
     * @see java.lang.Object#clone()
     */
    public Object clone()
        throws CloneNotSupportedException
    {
        Selection newSel = new Selection();

        for(Iterator it = getElements().iterator(); it.hasNext();)
        {
            GraphElement ge = (GraphElement) it.next();
            newSel.add(ge);
        }

        return newSel;
    }

    /**
     * Removes the given node or edge from the selection.
     *
     * @param ge the node or edge to remove from the selection.
     */
    public void remove(GraphElement ge)
    {
        if(ge instanceof Node)
        {
            this.nodes.remove(ge);
        }
        else
        {
            this.edges.remove(ge);
        }

        newMarked.remove(ge);
        this.newUnmarked.put(ge, null);
    }

	public boolean contains(GraphElement graphElement) {
		return (nodes.contains(graphElement) || edges.contains(graphElement));
	}

    /**
     * Gets a string describing the selection. Default: number of selected
     * nodes and edges in a sentence.
     *
     * @return a string describing the selection. Default: number of selected
     *         nodes and edges in a sentence.
     */
    public String toString()
    {
        return nodes.size() + " nodes and " + edges.size() + " edges selected";
    }

    /**
     * Clears the maps holding any changes since the last selectionChanged
     * event. Should be called whenever a selectionChanged event has been
     * generated with this selection.
     */
    protected void committedChanges()
    {
        this.newMarked = new HashMap();
        this.newUnmarked = new HashMap();
    }
}

//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
