//==============================================================================
//
//   GraphElementsDeletionEdit.java
//
//   Copyright (c) 2001-2003 Graffiti Team, Uni Passau
//
//==============================================================================
// $Id: GraphElementsDeletionEdit.java,v 1.5 2003/11/11 13:12:03 forster Exp $

package org.graffiti.undo;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;

import org.graffiti.graph.Edge;
import org.graffiti.graph.Graph;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;

/**
 * <code>GraphElementsDeletionEdit</code> makes deletion of graph elements
 * undoable.
 *
 * @author $Author $
 * @version $Revision: 1.5 $
 */
public class GraphElementsDeletionEdit
    extends GraphElementsEdit
{
    //~ Static fields/initializers =============================================

    /** The logger for the current class. */
    private static final Logger logger = Logger.getLogger(GraphElementsDeletionEdit.class.getName());

    //~ Instance fields ========================================================

    /**
     * this flag assures that execute method will be invoked before  calling
     * undo or redo methods
     */
    protected boolean executed = false;

    /** set of graph elements that were selected for deletion. */
    private LinkedHashSet graphElements;

    /**
     * temporary graph element set. It is necessary for update of graph
     * elements  contained in the selection set.
     */
    private Set tempGraphElements;

    //~ Constructors ===========================================================

    /**
     * @see GraffitiAbstractUndoableEdit#GraffitiAbstractUndoableEdit(Map)
     */
    public GraphElementsDeletionEdit(Collection graphElemList, Graph graph,
        Map geMap)
    {
        super(graph, geMap);
        this.graphElements = new LinkedHashSet(graphElemList);
        tempGraphElements = new HashSet();
    }

    //~ Methods ================================================================

    /**
     * Used to display the name for this edit.
     *
     * @return the name of this edit.
     *
     * @see javax.swing.undo.UndoableEdit
     */
    public String getPresentationName()
    {
        String name = "";

        if(graphElements.size() == 1)
        {
            if(graphElements.iterator().next() instanceof Node)
            {
                name = sBundle.getString("undo.deleteNode");
            }
            else if(graphElements.iterator().next() instanceof Edge)
            {
                name = name = sBundle.getString("undo.deleteEdge");
            }
        }
        else if(graphElements.size() > 1)
        {
            name = sBundle.getString("undo.deleteGraphElements");
        }

        return name;
    }

    /**
     * Executes the deletion of selected grpah elements
     */
    public void execute()
    {
        executed = true;

        /* saves adjacent edges of nodes in the common graph elements set.*/
        Set adjacentEdges = new HashSet();

        for(Iterator iter = graphElements.iterator(); iter.hasNext();)
        {
            GraphElement ge = (GraphElement) iter.next();

            if(ge instanceof Node)
            {
                Collection edgeList = ((Node) ge).getEdges();
                adjacentEdges.addAll(edgeList);
            }
        }

        graphElements.addAll(adjacentEdges);

        /* deletes all edges contained in the selection set firstly.*/
        for(Iterator iter = graphElements.iterator(); iter.hasNext();)
        {
            GraphElement ge = (GraphElement) iter.next();

            if(ge instanceof Edge)
            {
                graph.deleteEdge((Edge) ge);
            }
        }

        /* deletes all nodes contained in the selection set then.*/
        for(Iterator iter = graphElements.iterator(); iter.hasNext();)
        {
            GraphElement ge = (GraphElement) iter.next();

            if(ge instanceof Node)
            {
                graph.deleteNode((Node) ge);
            }
        }
    }

    /**
     * Deletes the GraphElements stored in this edit.
     */
    public void redo()
    {
        super.redo();

        if(!executed)
        {
            logger.info("The execute method hasn't been invocated");

            return;
        }

        for(Iterator iter = graphElements.iterator(); iter.hasNext();)
        {
            GraphElement ge = (GraphElement) iter.next();

            if(ge instanceof Edge)
            {
                Edge newEdge = (Edge) getCurrentGraphElement(ge);
                tempGraphElements.add(newEdge);
                graph.deleteEdge(newEdge);
            }
        }

        for(Iterator iter = graphElements.iterator(); iter.hasNext();)
        {
            GraphElement ge = (GraphElement) iter.next();

            if(ge instanceof Node)
            {
                Node newNode = (Node) getCurrentGraphElement(ge);
                tempGraphElements.add(newNode);
                graph.deleteNode(newNode);
            }
        }

        /* updates all graph elements referencies in the set containing graph
         * elements.
         */
        graphElements.clear();
        graphElements.addAll(tempGraphElements);
        tempGraphElements.clear();
    }

    /**
     * Adds the deleted GraphElements stored in this edit.
     */
    public void undo()
    {
        super.undo();

        if(!executed)
        {
            logger.info("The execute method hasn't been invocated");

            return;
        }

        for(Iterator iter = graphElements.iterator(); iter.hasNext();)
        {
            GraphElement ge = (GraphElement) iter.next();

            if(ge instanceof Node)
            {
                Node newNode = graph.addNodeCopy((Node) ge);
                geMap.put((Node) ge, newNode);
            }
        }

        for(Iterator iter = graphElements.iterator(); iter.hasNext();)
        {
            GraphElement ge = (GraphElement) iter.next();

            if(ge instanceof Edge)
            {
                logger.info("undo the edge deleting");

                Edge oldEdge = (Edge) ge;

                assert (oldEdge.getSource() != null);
                assert (oldEdge.getTarget() != null);

                Node source = (Node) getCurrentGraphElement(oldEdge.getSource());
                Node target = (Node) getCurrentGraphElement(oldEdge.getTarget());
                assert (source != null);
                assert (target != null);
                assert (source.getGraph() == graph);
                assert (target.getGraph() == graph);
                assert (oldEdge.getGraph() == graph);

                Edge newEdge = graph.addEdgeCopy(oldEdge, source, target);

                geMap.put(oldEdge, newEdge);
            }
        }
    }
}

//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
